# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-10-31 20:24
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Event',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('event_id', models.CharField(db_index=True, max_length=255)),
                ('event_type', models.CharField(max_length=255)),
                ('event_data', models.TextField()),
                ('aggregate_id', models.CharField(db_index=True, max_length=255)),
                ('aggregate_type', models.CharField(max_length=255)),
                ('aggregate_version', models.IntegerField()),
                ('create_date', models.DateTimeField()),
                ('metadata', models.TextField()),
                ('module_name', models.CharField(db_column='_module_name', max_length=255)),
                ('class_name', models.CharField(db_column='_class_name', max_length=255)),
                ('stored_entity_id', models.CharField(db_column='_stored_entity_id', max_length=255)),
            ],
            options={
                'db_table': 'event_journal',
            },
        ),
        migrations.AlterUniqueTogether(
            name='event',
            unique_together=set([('aggregate_id', 'aggregate_type', 'aggregate_version')]),
        ),
    ]
